from __future__ import annotations
import os
import tempfile
import requests
from PIL import Image
from loguru import logger
from urllib.parse import urlparse
from dataclasses import dataclass


@dataclass
class RadarSatellite:
    imsradar_images: list
    radar_images: list
    middle_east_satellite_images: list
    europe_satellite_images: list

    def __init__(
        self,
        imsradar_images: list = [],
        radar_images: list = [],
        middle_east_satellite_images: list = [],
        europe_satellite_images: list = [],
    ):
        self.imsradar_images = imsradar_images
        self.radar_images = radar_images
        self.middle_east_satellite_images = middle_east_satellite_images
        self.europe_satellite_images = europe_satellite_images

    def generate_images(self, path: str = ""):
        self.create_animation("imsradar.gif", self.imsradar_images, path)
        self.create_animation("radar.gif", self.radar_images, path)
        self.create_animation(
            "middle_east.gif", self.middle_east_satellite_images, path
        )
        self.create_animation("europe.gif", self.europe_satellite_images, path)

    def create_animation(self, animated_file: str, images: list, path: str):
        """
        This method will download the images needed to create animated Radar / Satellite image
        parameters:
            >>> path: path to save the animated image. if path wil not be provided, the default path will be the current one.
            >>> animated_file: The name of the animated file
            >>> images: the list of images for creating the animation.
        """
        try:
            if os.path.exists(path):
                animated_image_path = path + "/" + animated_file
            else:
                animated_image_path = (
                    os.path.realpath(os.path.dirname(__file__)) + "/" + animated_file
                )
            logger.debug(
                "Creating " + animated_file + " animation at: " + animated_image_path
            )

            for idx, item in enumerate(images):
                file = requests.get(images[idx])
                open(
                    tempfile.gettempdir()
                    + "/"
                    + os.path.basename(urlparse(images[idx]).path),
                    "wb",
                ).write(file.content)
                images[idx] = (
                    tempfile.gettempdir()
                    + "/"
                    + os.path.basename(urlparse(images[idx]).path)
                )

            frames = [Image.open(image) for image in images]
            frame_one = frames[0]
            frame_one.save(
                animated_image_path,
                format="GIF",
                append_images=frames,
                save_all=True,
                duration=4,
                loop=0,
            )

            for image in images:
                os.remove(image)
            return animated_image_path
        except Exception as e:
            logger.error("Error creating " + animated_file + " animation. " + str(e))
            return None



import json
from datetime import datetime
from typing import Type, Optional

import requests
from loguru import logger
from weatheril.consts import EN_LOCATIONS, EN_WEATHER_CODES, EN_WIND_DIRECTIONS, HE_LOCATIONS, HE_WEATHER_CODES, HE_WIND_DIRECTIONS, LOCATIONS_INFO_URL, WARNINGS_METADTA_URL, WEATHER_CODES_URL, WIND_DIRECTIONS_URL, WEEKDAY_NAMES
from weatheril.consts import REGIONS_URL
from weatheril.consts import SEA_REGIONS_URL

# ims.gov.il does not support ipv6 yet, `requests` use ipv6 by default
# and wait for timeout before trying ipv4, so we have to disable ipv6
requests.packages.urllib3.util.connection.HAS_IPV6 = False

_weather_code_map = {}
_locations_map = {}
_wind_direction_map = {}
_regions_map = {}
_sea_regions_map = {}
_warning_type_map = {}
_warning_group_map = {}
_warning_severity_map = {}

def get_weather_description_by_code(language: str, code: int | None) -> str:
    """
    Get the weather description by the weather code
    """
    global _weather_code_map
    if not _weather_code_map:
        _weather_code_map = _get_weather_codes(language)
    if not code:
        return "Nothing"
    code = int(code)
    return _weather_code_map.get(code, "Nothing")

def _get_weather_codes(language) -> dict:
    """
    Get the weather codes from IMS
    """
    try:
        url = WEATHER_CODES_URL.format(language=language)
        data = fetch_data(url)
        return {int(d["id"]): d["desc"] for d in data["data"].values()}
    except Exception as e:
        logger.error("Error getting weather codes. " + str(e))
        logger.exception(e)
        return HE_WEATHER_CODES if language == "he" else EN_WEATHER_CODES


def get_location_name_by_id(language: str, lid: str | int):
    """
    Converts location id to City name
    """
    lid = int(lid)
    global _locations_map
    if not _locations_map:
        _locations_map = _get_locations_map(language)
    location = _locations_map.get(lid)
    return location["name"] if location else "Nothing"

def get_location_info_by_id(language: str, lid: str | int):
    """
    Converts location id to City name
    """
    lid = int(lid)
    global _locations_map
    if not _locations_map:
        _locations_map = _get_locations_map(language)
    return _locations_map.get(int(lid))

def _get_locations_map(language) -> dict:
    """
    Get the location information from IMS
    """
    try:
        url = LOCATIONS_INFO_URL.format(language=language)
        data = fetch_data(url)
        return {int(d["lid"]): d for d in data["data"].values()}
    except Exception as e:
        logger.error("Error getting locations info.. " + str(e))
        logger.exception(e)
        return HE_LOCATIONS if language == "he" else EN_LOCATIONS


def get_wind_direction(language: str, direction_code: int) -> int:
    """
    Converts the wind direction code to azimuth
    """
    global _wind_direction_map
    if not _wind_direction_map:
        _wind_direction_map = _get_wind_direction_map(language)
    direction = _wind_direction_map.get(direction_code)
    if not direction:
        return -1
    if isinstance(direction, int):
        return direction
    return int(direction.get("direction"))

def _get_wind_direction_map(language) -> dict:
    """
    Get the Wind Direction Map from IMS
    """
    try:
        url = WIND_DIRECTIONS_URL.format(language=language)
        data = fetch_data(url)
        return {int(k): v for k, v in data["data"].items()}
    except Exception as e:
        logger.error("Error getting directions info.. " + str(e))
        logger.exception(e)
        return HE_WIND_DIRECTIONS if language == "he" else EN_WIND_DIRECTIONS

def get_sea_region_by_id(language: str, region_id: int):
    """
    Get Region Information by Id
    """
    global _sea_regions_map
    if not _sea_regions_map:
        _sea_regions_map = _get_sea_regions(language)
    if not _sea_regions_map:
        return None
    sea_region = _sea_regions_map.get(region_id)
    return sea_region


def _get_sea_regions(language) -> dict:
    """
    Get the Sea Regions Map from IMS
    """
    try:
        url = SEA_REGIONS_URL.format(language=language)
        data = fetch_data(url)
        return {int(v['rid']): v for v in data["data"].values()}
    except Exception as e:
        logger.error("Error getting directions info.. " + str(e))
        logger.exception(e)
        raise e


def get_region_by_id(language: str, region_id: str) -> dict:
    """
    Get Region Information by Id
    """
    global _regions_map
    if not _regions_map:
        _regions_map = _get_regions(language)
    if not _regions_map:
        return {}
    region = _regions_map.get(region_id, {})
    return region


def _get_regions(language) -> dict | None:
    """
    Get the Regions Map from IMS
    """
    try:
        url = REGIONS_URL.format(language=language)
        data = fetch_data(url)
        return {v["rid"]: v for v in data["data"]}
    except Exception as e:
        logger.error("Error getting Regions info.. " + str(e))
        logger.exception(e)
        raise e


def _get_warning_metadata(language) -> dict | None:
    """
    Get the Warning Types Map from IMS
    """
    try:
        url = WARNINGS_METADTA_URL.format(language=language)
        data = fetch_data(url)
        return data["data"]
    except Exception as e:
        logger.error("Error getting Warning Metadata... " + str(e))
        logger.exception(e)
        raise e

def get_warning_type_by_id(language: str, warning_type_id: int) -> dict:
    """
    Get the Warning Types by Id
    """
    global _warning_type_map
    global _warning_group_map
    global _warning_severity_map
    if not _warning_type_map:
        _warning_metadata = _get_warning_metadata(language)
        if _warning_metadata is None:
            raise ValueError("Warning Type Map not found")

        _warning_type_map = {int(v["warning_type_id"]): v for v in _warning_metadata["ims_warning_type"].values()}
        _warning_group_map = {k: v for k,v in _warning_metadata["warning_groups"].items()}
        _warning_severity_map = {int(v["severity_id"]): v for v in _warning_metadata["warning_severity"].values()}
    if not _warning_type_map:
        raise ValueError("Warning Type Map not found")
    return _warning_type_map.get(warning_type_id, {})

def get_warning_group_by_id(language: str, warning_group_id: str) -> dict:
    """
    Get the Warning Group by Id
    """
    global _warning_type_map
    global _warning_group_map
    global _warning_severity_map
    if not _warning_type_map:
        _warning_metadata = _get_warning_metadata(language)
        if _warning_metadata is None:
            return {}

        _warning_type_map = {int(v["warning_type_id"]): v for v in _warning_metadata["ims_warning_type"].values()}
        _warning_group_map = {k: v for k,v in _warning_metadata["warning_groups"].items()}
        _warning_severity_map = {int(v["severity_id"]): v for v in _warning_metadata["warning_severity"].values()}
    if not _warning_type_map:
        raise ValueError("Warning Group Map not found")
    return _warning_group_map.get(warning_group_id, {})

def get_warning_severity_by_id(language: str, warning_severity_id: int) -> dict:
    """
    Get the Warning Severity by Id
    """
    global _warning_type_map
    global _warning_group_map
    global _warning_severity_map
    if not _warning_type_map:
        _warning_metadata = _get_warning_metadata(language)
        if _warning_metadata is None:
            return {}

        _warning_type_map = {int(v["warning_type_id"]): v for v in _warning_metadata["ims_warning_type"].values()}
        _warning_group_map = {k: v for k,v in _warning_metadata["warning_groups"].items()}
        _warning_severity_map = {int(v["severity_id"]): v for v in _warning_metadata["warning_severity"].values()}
    if not _warning_severity_map:
        raise ValueError("Warning Severity Map not found")
    return _warning_severity_map.get(warning_severity_id, {})

def get_day_of_the_week(language: str, date: datetime):
    """
    Converts the given date to day of the week name
    """
    day = date.strftime("%A")
    if language == "he":
        return WEEKDAY_NAMES.get(day, "nothing")
    else:
        return day

def get_value(
    data: dict,
    key: str,
    inner_dict_key: Optional[str],
    data_type: Type = dict,
    default_value=None,
    custom_empty_value=None
):
    """
    Get default value from nested dictionary and convert to specified data type
    :param data: dictionary
    :param key: first level key
    :param inner_dict_key: second level key
    :param data_type: data type to convert to (str, int, float)
    :param default_value: default value
    :return: data[key][dict_key] or data[key] or default_value
    """
    value = None
    if key in data:
        if inner_dict_key is None:
            value = data.get(key)
        elif isinstance(data.get(key), dict) and inner_dict_key in data.get(key):
            value = data.get(key).get(inner_dict_key)

    if value is None:
        return default_value

    try:
        if data_type is str:
            value = str(value)
        if data_type is int:
            value = int(value)
        if data_type is float:
            value = float(value)
    except (ValueError, TypeError):
        value = default_value

    if custom_empty_value and value == custom_empty_value:
        return default_value
    return value


def fetch_data(url: str) -> dict:
    """
    Helper method to get the Json data from ims website
    """
    try:
        logger.debug("Getting data from: " + url)
        response = requests.get(url)
        response = json.loads(response.text)
        return response
    except Exception as e:
        logger.error("Error getting data. " + str(e))
        logger.exception(e)
        return dict()


def get_data(current_data, url, last_fetch_time, cache_expiration_in_sec) -> dict:
    if (
        current_data
        and (datetime.now() - last_fetch_time).total_seconds()
        < cache_expiration_in_sec
    ):
        return current_data
    try:
        logger.debug("Getting data from " + url)
        return fetch_data(url).get("data", {})
    except Exception as e:
        logger.error("Error getting city portal data. " + str(e))
        logger.exception(e)
    return {}



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from .consts import TIMEZONE
from .utils import get_warning_severity_by_id, get_warning_type_by_id, get_region_by_id, get_warning_group_by_id, get_location_info_by_id


@dataclass
class Warning:
    language: str
    location_id: int
    wid: int
    alert_id: int
    severity_id: int
    warning_type_id: int
    sent: str | datetime
    valid_from: str | datetime
    valid_to: str | datetime
    full_en: str
    full_he: str
    text: str
    text_full: str
    valid_from_unix: int
    groups: list[int | str]
    regions: list[int | str]
    region_name: str = field(init=False)
    severity: str = field(init=False)
    warning_type: str = field(init=False)


    def __post_init__(self):
        location_info = get_location_info_by_id(self.language, self.location_id)
        if not location_info:
            raise ValueError(f"Location not found for id: {self.location_id}")

        rid = location_info.get('rid')
        region = get_region_by_id(self.language, region_id="r-" + str(rid))
        self.region_name = region.get("name", "")

        self.severity = get_warning_severity_by_id(self.language, self.severity_id).get("severity_name", "")
        self.warning_type = get_warning_type_by_id(self.language, int(self.warning_type_id)).get("name", "")
        self.sent = TIMEZONE.localize(datetime.strptime(self.sent, "%Y-%m-%d %H:%M:%S")) if isinstance(self.sent, str) else self.sent
        self.valid_from = TIMEZONE.localize(datetime.strptime(self.valid_from, "%Y-%m-%d %H:%M:%S")) if isinstance(self.valid_from, str) else self.valid_from
        self.valid_to = TIMEZONE.localize(datetime.strptime(self.valid_to, "%Y-%m-%d %H:%M:%S")) if isinstance(self.valid_to, str) else self.valid_to

        self.groups = list(map(lambda gid: get_warning_group_by_id(self.language, "g-" + str(gid))["name"], self.groups))

        self.regions = list(map(lambda rid: get_region_by_id(self.language, "r-" + str(rid)).get("name", ""), self.regions))

        if not self.text_full:
            self.text_full = self.full_en.strip() if self.language == "en" else self.full_he.strip()



from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from .utils import (
    get_location_name_by_id,
    get_weather_description_by_code,
    get_wind_direction,
)


@dataclass
class Weather:
    language: str
    lid: str
    humidity: int
    rain: float
    rain_chance: int
    temperature: float
    due_point_temp: int
    wind_speed: int
    wind_chill: int
    wind_direction_id: int
    feels_like: float
    heat_stress_level: int
    forecast_time: datetime
    u_v_index: int
    u_v_level: str
    u_v_i_max: Optional[int]
    u_v_i_factor: Optional[float]
    min_temp: Optional[int]
    max_temp: Optional[int]
    pm10: int
    json: str
    weather_code: Optional[int]
    wave_height: float
    location: str = field(init=False)
    description: str = field(init=False)
    wind_direction: int = field(init=False)
    gust_speed: Optional[int]
    modified_at: datetime

    def __post_init__(self):
        self.location = get_location_name_by_id(self.language, self.lid)
        self.description = get_weather_description_by_code(
            self.language, self.weather_code
        )
        self.wind_direction = get_wind_direction(self.language, self.wind_direction_id)



"""Israel Meteorological Service unofficial python api wrapper"""
from datetime import datetime

import requests
from loguru import logger

from .consts import CURRENT_ANALYSIS_URL, FORECAST_URL, IMS_API_URL_BASE, RADAR_SATELLITE_URL, WARNINGS_URL, TIMEZONE
from .forecast import Forecast, Daily, Hourly
from .radar_satellite import RadarSatellite
from .warning import Warning
from .utils import get_region_by_id, get_value, fetch_data, get_data, get_location_info_by_id
from .weather import Weather


# ims.gov.il does not support ipv6 yet, `requests` use ipv6 by default
# and wait for timeout before trying ipv4, so we have to disable ipv6
requests.packages.urllib3.util.connection.HAS_IPV6 = False

DAILY_KEY = "daily"
HOURLY_KEY = "hourly"
FULL_WARNINGS_DATA_KEY = "full_warnings_data"


DEFAULT_CACHE_EXPIRATION = 30


class WeatherIL:
    def __init__(
        self, location, language="he", cache_expiration_in_sec=DEFAULT_CACHE_EXPIRATION
    ):
        """
        Init the WeatherIL object.
        parameters:
            >>> location: Location Id for the forecast (Table exists in the readme)
            >>> language: can be he (Hebrew) or en (English). default will be "he"
            >>> city_portal_cache_expiration: cache expiration in days for city portal data API. default is 30 seconds
        """
        self._cache_expiration_in_sec = cache_expiration_in_sec
        self.language = language
        self.location = str(location)
        self._analysis_data = None
        self._analysis_last_fetch = None
        self._forecast_data = None
        self._forecast_last_fetch = None
        self._full_warnings_data = None
        self._warnings_last_fetch = None

    def get_current_analysis(self):
        self._get_analysis_data()
        try:
            logger.debug("Getting current analysis")
            analysis_data = self._analysis_data.get(self.location, {})
            if analysis_data:
                logger.debug("Got current analysis for location " + str(self.location))
                # Parse forecast_time and modified_at separately due to datetime parsing
                forecast_time_str = get_value(
                    analysis_data, "forecast_time", None, str
                )
                forecast_time = (
                    TIMEZONE.localize(
                        datetime.strptime(forecast_time_str, "%Y-%m-%d %H:%M:%S")
                    )
                    if forecast_time_str
                    else None
                )

                modified_at_str = get_value(analysis_data, "modified", None, str)
                modified_at = (
                    TIMEZONE.localize(
                        datetime.strptime(modified_at_str, "%Y-%m-%d %H:%M:%S")
                    )
                    if modified_at_str
                    else None
                )

                return Weather(
                    language=self.language,
                    lid=get_value(analysis_data, "lid", None, str),
                    humidity=get_value(
                        analysis_data, "relative_humidity", None, int, 0
                    ),
                    rain=get_value(analysis_data, "rain", None, float, 0.0, -999.0),
                    rain_chance=get_value(analysis_data, "rain_chance", None, int, 0),
                    temperature=get_value(
                        analysis_data, "temperature", None, float, 0.0
                    ),
                    due_point_temp=get_value(
                        analysis_data, "due_point_Temp", None, int, 0
                    ),
                    wind_speed=get_value(analysis_data, "wind_speed", None, int, 0),
                    wind_chill=get_value(analysis_data, "wind_chill", None, int, 0),
                    wind_direction_id=get_value(
                        analysis_data, "wind_direction_id", None, int, 0
                    ),
                    feels_like=get_value(
                        analysis_data, "feels_like", None, float
                    ),
                    heat_stress_level=get_value(
                        analysis_data, "heat_stress_level", None, int, 0
                    ),
                    u_v_index=get_value(analysis_data, "u_v_index", None, int, 0),
                    u_v_level=get_value(analysis_data, "u_v_level", None, str),
                    u_v_i_max=get_value(analysis_data, "u_v_i_max", None, int),
                    u_v_i_factor=get_value(analysis_data, "u_v_i_factor", None, float),
                    wave_height=get_value(
                        analysis_data, "wave_height", None, float, 0.0
                    ),
                    max_temp=get_value(analysis_data, "max_temp", None, int),
                    min_temp=get_value(analysis_data, "min_temp", None, int),
                    pm10=get_value(analysis_data, "pm10", None, int, 0),
                    forecast_time=forecast_time,
                    modified_at=modified_at,
                    json=analysis_data,
                    weather_code=get_value(analysis_data, "weather_code", None, int),
                    gust_speed=get_value(analysis_data, "gust_speed", None, int, None, -999)
                )
            else:
                logger.error('No "' + self.location + '" in current analysis response')
                logger.debug("Response: " + analysis_data)
                return None
        except Exception as e:
            logger.error("Error getting current analysis.")
            logger.exception(e)
            return None

    def get_forecast(self):
        """
        Get weather forecast
        return: Forecast object
        """
        logger.debug("Getting forecast")
        self._get_forecast_data()
        try:
            days = []
            forecast_data = self._forecast_data
            logger.debug("Got forecast for location " + str(self.location))
            for key in forecast_data.keys():
                hours = self._get_hourly_forecast(
                    get_value(forecast_data, key, HOURLY_KEY, dict)
                )
                daily = Daily(
                    language=self.language,
                    date=TIMEZONE.localize(datetime.strptime(key, "%Y-%m-%d")),
                    lid=get_value(
                        forecast_data[key], DAILY_KEY, "lid", default_value="0"
                    ),
                    weather_code=get_value(
                        forecast_data[key], DAILY_KEY, "weather_code", int
                    ),
                    minimum_temperature=get_value(
                        forecast_data[key], DAILY_KEY, "minimum_temperature", int
                    ),
                    maximum_temperature=get_value(
                        forecast_data[key], DAILY_KEY, "maximum_temperature", int
                    ),
                    maximum_uvi=get_value(
                        forecast_data[key], DAILY_KEY, "maximum_uvi", int
                    ),
                    u_v_i_factor=get_value(
                        forecast_data[key], "daily", "u_v_i_factor", float
                    ),
                    hours=hours,
                    description=(
                        get_value(
                            forecast_data[key],
                            "country",
                            "description",
                            default_value="",
                        )
                    ).rstrip(),
                )
                days.append(daily)
            return Forecast(days)

        except Exception as e:
            logger.error("Error getting forecast data")
            logger.exception(e)
            return None

    def _get_hourly_forecast(self, data):
        """
        Get the hourly forecast
        """
        hours = []
        try:
            for key in data.keys():
                hours.append(
                    Hourly(
                        language=self.language,
                        hour=key,
                        forecast_time=TIMEZONE.localize(
                            datetime.strptime(
                                data.get(key, {}).get("forecast_time"),
                                "%Y-%m-%d %H:%M:%S",
                            )
                        ),
                        created=TIMEZONE.localize(
                            datetime.strptime(
                                data.get(key, {}).get("created"), "%Y-%m-%d %H:%M:%S"
                            )
                        ),
                        weather_code=get_value(data, key, "weather_code", int),
                        temperature=get_value(data, key, "temperature", int),
                        precise_temperature=get_value(
                            data, key, "precise_temperature", float
                        ),
                        heat_stress=get_value(data, key, "heat_stress", float),
                        heat_stress_level=get_value(
                            data, key, "heat_stress_level", int
                        ),
                        pm10=get_value(data, key, "pm10", int),
                        relative_humidity=get_value(
                            data, key, "relative_humidity", int
                        ),
                        rain=get_value(data, key, "rain", float, None, -999.0),
                        rain_chance=get_value(data, key, "rain_chance", int),
                        wind_speed=get_value(data, key, "wind_speed", int),
                        gust_speed=get_value(data, key, "gust_speed", int, None, -999),
                        wind_direction_id=get_value(
                            data, key, "wind_direction_id", int
                        ),
                        wave_height=get_value(data, key, "wave_height", float),
                        wind_chill=get_value(data, key, "wind_chill", int),
                        u_v_index=get_value(data, key, "u_v_index", int, None, -8991),
                        u_v_i_max=get_value(data, key, "u_v_i_max", int),
                    )
                )
            return hours
        except Exception as e:
            logger.error("Error getting hourly forecast ")
            logger.exception(e)
            return None

    def get_radar_images(self):
        """
        Get the list of images for Satellite and Radar
        return: RadarSatellite objects with the lists
        """
        rs = RadarSatellite()
        try:
            logger.debug("Getting radar images")
            url = RADAR_SATELLITE_URL.format(language=self.language)
            data = fetch_data(url)
            base_url = IMS_API_URL_BASE.format(language="").rstrip("/")
            for key in data.get("data").get("types").get("IMSRadar"):
                rs.imsradar_images.append(base_url + key.get("file_name"))

            for key in data.get("data").get("types").get("radar"):
                rs.radar_images.append(base_url + key.get("file_name"))

            for key in data.get("data").get("types").get("MIDDLE-EAST"):
                rs.middle_east_satellite_images.append(
                    base_url + key.get("file_name")
                )

            for key in data.get("data").get("types").get("EUROPE"):
                rs.europe_satellite_images.append(base_url + key.get("file_name"))

            logger.debug(f"\
                Got: {len(rs.imsradar_images)} IMS Radar Images;\
                {len(rs.radar_images)} Radar Images;\
                {len(rs.middle_east_satellite_images)} Middle East Satellite Images;\
                {len(rs.europe_satellite_images)} European Satellite Images")
            return rs
        except Exception as e:
            logger.error("Error getting images. " + str(e))
            return rs

    def _get_analysis_data(self):
        """
        Get the city current analysis data
        return: dict
        """
        url = CURRENT_ANALYSIS_URL.format(language=self.language, location=self.location)
        self._analysis_data = get_data(
            self._analysis_data, url, self._analysis_last_fetch, self._cache_expiration_in_sec
        )
        if self._analysis_data:
            self._analysis_last_fetch = datetime.now()

    def _get_forecast_data(self):
        """
        Get the city forecast data
        """
        url = FORECAST_URL.format(language=self.language, location=self.location)
        self._forecast_data = get_data(
            self._forecast_data, url, self._forecast_last_fetch, self._cache_expiration_in_sec
        )
        if self._forecast_data:
            self._forecast_last_fetch = datetime.now()

    def _get_warnings_data(self):
        """
        Get the all warning data
        """

        url = WARNINGS_URL.format(language=self.language)
        self._full_warnings_data = get_data(
            self._full_warnings_data, url, self._warnings_last_fetch, self._cache_expiration_in_sec
        )

        if self._full_warnings_data:
            self._warnings_last_fetch = datetime.now()

    def get_warnings(self):
        """
        Get weather forecast
        return: Forecast object
        """
        logger.debug("Getting warnings")
        self._get_warnings_data()

        location_info = get_location_info_by_id(self.language, self.location)
        if not location_info:
            raise ValueError(f"Location not found for id {self.location}")

        rid = location_info.get('rid')
        region = get_region_by_id(self.language, region_id="r-" + rid)
        if not region:
            raise ValueError(f"Region not found for id {rid}")

        warnings = []
        if  self._full_warnings_data:
            for key in self._full_warnings_data[FULL_WARNINGS_DATA_KEY]:
                daily_warnings: dict = get_value(self._full_warnings_data, FULL_WARNINGS_DATA_KEY, key, dict)
                regional_alerts = daily_warnings.get("r-" + rid, {})
                for alert in regional_alerts.values():

                    warnings.append(Warning(
                        language=self.language,
                        location_id=int(self.location),
                            wid=int(alert["wid"]),
                            alert_id=int(alert["alert_id"]),
                            severity_id=int(alert["severity_id"]),
                            warning_type_id=int(alert["warning_type_id"]),
                            sent=alert["sent"],
                            valid_from=alert["valid_from"],
                            valid_to=alert["valid_to"],
                            full_en=alert["full_en"],
                            full_he=alert["full_he"],
                            text=alert["text"],
                            text_full=alert["text_full"],
                            valid_from_unix=int(alert["valid_from_unix"]),
                            groups=alert["groups"],
                            regions=alert["regions"]
                    ))

        return warnings
